# -*- coding: utf-8 -*-

"""
forcefield_step
A step for choosing the default forcefield.
"""

# Bring up the classes so that they appear to be directly in
# the package.

from forcefield_step.forcefield import Forcefield  # noqa: F401
from forcefield_step.forcefield_parameters import ForcefieldParameters  # noqa: F401,E501
from forcefield_step.forcefield_step import ForcefieldStep  # noqa: F401
from forcefield_step.tk_forcefield import TkForcefield  # noqa: F401

# Handle versioneer
from ._version import get_versions
__author__ = """Paul Saxe"""
__email__ = 'psaxe@molssi.org'
versions = get_versions()
__version__ = versions['version']
__git_revision__ = versions['full-revisionid']
del get_versions, versions



# -*- coding: utf-8 -*-
"""Control parameters for using forcefields
"""

import importlib
import logging
import os
import pkg_resources
import seamm

logger = logging.getLogger(__name__)


# Get the list of available forcefields
path = pkg_resources.resource_filename(__name__, 'data/')

# Check if we can use OpenKIM
if importlib.util.find_spec('kim_query') is None:
    forcefields = []
else:
    forcefields = ['OpenKIM']

logger.debug('Looking for forcefields at ' + path)
with os.scandir(path) as it:
    for entry in it:
        if not entry.name.startswith('.') and entry.is_file():
            logger.debug('   ' + entry.name)
            ffname, ext = os.path.splitext(entry.name)
            if ext == '.frc':
                forcefields.append(entry.name)

forcefields = sorted(forcefields)


class ForcefieldParameters(seamm.Parameters):
    """The control parameters for forcefields"""

    parameters = {
        "task": {
            "default": 'setup forcefield',
            "kind": "enumeration",
            "default_units": "",
            "enumeration": (
                'setup forcefield',
                'assign forcefield to structure',
            ),
            "format_string": "s",
            "description": "What to do:",
            "help_text": "What to do with the forcefield."
        },
        "forcefield_file": {
            "default": forcefields[0],
            "kind": "enumeration",
            "default_units": "",
            "enumeration": tuple(forcefields),
            "format_string": "s",
            "description": "Forcefield Repository:",
            "help_text": "The forcefield repository or file to use."
        },
        "forcefield": {
            "default": "default",
            "kind": "enumeration",
            "default_units": "",
            "enumeration": ('default',),
            "format_string": "s",
            "description": "Forcefield:",
            "help_text": "The forcefield with the file."
        },
        "elements": {
            "default": "",
            "kind": "periodic table",
            "default_units": None,
            "enumeration": None,
            "format_string": "",
            "description": "Elements:",
            "help_text": "The elements to include."
        },
        "potentials": {
            "default": "",
            "kind": "enumeration",
            "default_units": "",
            "enumeration": ('will be replaced',),
            "format_string": "s",
            "description": "Interatomic Potentials:",
            "help_text": "The interatomic potentials to use."
        },
    }

    def __init__(self, defaults={}, data=None):
        """Initialize the instance, by default from the default
        parameters given in the class"""

        super().__init__(
            defaults={**ForcefieldParameters.parameters, **defaults},
            data=data
        )



# -*- coding: utf-8 -*-

"""The graphical part of a Forcefield step"""

import seamm
import tkinter as tk

try:
    import kim_query
except ModuleNotFoundError:
    pass


class TkForcefield(seamm.TkNode):
    """The graphical part of a forcefield step
    """

    def __init__(
        self,
        tk_flowchart=None,
        node=None,
        canvas=None,
        x=None,
        y=None,
        w=200,
        h=50
    ):
        '''Initialize a node

        Keyword arguments:
        '''

        self.dialog = None

        super().__init__(
            tk_flowchart=tk_flowchart,
            node=node,
            canvas=canvas,
            x=x,
            y=y,
            w=w,
            h=h
        )

    def right_click(self, event):
        """Probably need to add our dialog...
        """

        super().right_click(event)
        self.popup_menu.add_command(label="Edit..", command=self.edit)

        self.popup_menu.tk_popup(event.x_root, event.y_root, 0)

    def edit(self):
        """Present a dialog for editing this step's parameters.
        We need to recreate the PeriodicTable for some reason
        """
        if self.dialog is None:
            self.create_dialog()

        # Reset the dialog, which will rectreate the PeriodicTable
        # widget
        self.reset_dialog()
        # And resize the dialog to fit...
        self.fit_dialog()

        super().edit()

    def create_dialog(self):
        """Create the dialog for editing the Forcefield flowchart
        """

        frame = super().create_dialog('Edit Forcefield Step')

        # Create the widgets
        P = self.node.parameters
        for key in P:
            self[key] = P[key].widget(frame)

        # bindings...
        self['task'].combobox.bind("<<ComboboxSelected>>", self.reset_dialog)
        self['task'].config(state='readonly')

        self['forcefield_file'].combobox.bind(
            "<<ComboboxSelected>>", self.reset_dialog
        )
        self['forcefield_file'].combobox.bind("<Return>", self.reset_dialog)
        self['forcefield_file'].combobox.bind("<FocusOut>", self.reset_dialog)

        # and set it up the first time
        self.reset_dialog()

    def reset_dialog(self, widget=None):
        """Layout the widgets as needed for the current state"""

        frame = self['frame']
        for slave in frame.grid_slaves():
            slave.grid_forget()

        task = self['task'].get()
        repository = self['forcefield_file'].get()

        row = 0
        self['task'].grid(row=row, column=0, sticky=tk.W)
        row += 1

        if task == 'assign forcefield to structure':
            pass
        elif task == 'setup forcefield':
            self['forcefield_file'].grid(row=row, column=0, sticky=tk.W)
            row += 1
            if repository == 'OpenKIM':
                # For reasons unknown, the PeriodicTable does not redisplay
                # properly, so recreate.
                self['elements'].destroy()
                P = self.node.parameters
                self['elements'] = P['elements'].widget(
                    frame, command=self.update_potentials
                )
                self['elements'].grid(row=row, column=0, columnspan=2)
                frame.rowconfigure(row, weight=1)
                frame.columnconfigure(1, weight=1)
                row += 1
                self['potentials'].grid(
                    row=row, column=0, columnspan=2, sticky=tk.EW
                )
            else:
                self['forcefield'].grid(
                    row=row, column=0, columnspan=2, sticky=tk.EW
                )
                row += 1

        return row

    def update_potentials(self, elements):
        """Update the list of possible OpenKIM potentials as the element
        selection changes."""

        potentials = kim_query.get_available_models(elements)

        current_potential = self['potentials'].get()
        self['potentials'].combobox.configure(value=potentials)
        if current_potential not in potentials:
            if len(potentials) == 0:
                self['potentials'].set('')
            else:
                self['potentials'].set(potentials[0])



# -*- coding: utf-8 -*-

"""A node or step for the forcefield in a flowchart"""

import forcefield_step
import configargparse
import logging
import os.path
import pkg_resources
import seamm_ff_util
import seamm
import seamm.data as data
import seamm_util.printing as printing
from seamm_util.printing import FormattedText as __

logger = logging.getLogger(__name__)
job = printing.getPrinter()
printer = printing.getPrinter('forcefield')


def upcase(string):
    """Return an uppercase version of the string.

    Used for the type argument in argparse/
    """
    return string.upper()


class Forcefield(seamm.Node):

    def __init__(self, flowchart=None, extension=None):
        '''Initialize a forcefield step

        Keyword arguments:
        '''
        logger.debug('Creating Forcefield {}'.format(self))

        # Argument/config parsing
        self.parser = configargparse.ArgParser(
            auto_env_var_prefix='',
            default_config_files=[
                '/etc/seamm/forcefield_step.ini',
                '/etc/seamm/seamm.ini',
                '~/.seamm/forcefield_step.ini',
                '~/.seamm/seamm.ini',
            ]
        )

        self.parser.add_argument(
            '--seamm-configfile',
            is_config_file=True,
            default=None,
            help='a configuration file to override others'
        )

        # Options for this plugin
        self.parser.add_argument(
            "--from-smiles-log-level",
            default=configargparse.SUPPRESS,
            choices=[
                'CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG', 'NOTSET'
            ],
            type=upcase,
            help="the logging level for the From SMILES step"
        )

        self.options, self.unknown = self.parser.parse_known_args()

        # Set the logging level for this module if requested
        if 'forcefield_log_level' in self.options:
            logger.setLevel(self.options.forcefield_log_level)

        super().__init__(
            flowchart=flowchart, title='Forcefield', extension=extension
        )

        self.parameters = forcefield_step.ForcefieldParameters()

    @property
    def version(self):
        """The semantic version of this module.
        """
        return forcefield_step.__version__

    @property
    def git_revision(self):
        """The git version of this module.
        """
        return forcefield_step.__git_revision__

    def description_text(self, P=None):
        """Return a short description of this step.

        Return a nicely formatted string describing what this step will
        do.

        Keyword arguments:
            P: a dictionary of parameter values, which may be variables
                or final values. If None, then the parameters values will
                be used as is.
        """

        if not P:
            P = self.parameters.values_to_dict()

        if P['forcefield_file'][0] == '$':
            text = (
                "Read the forcefield file given in the variable"
                " '{forcefield_file}' and use the {forcefield} "
                "forcefield."
            )
        elif P['forcefield_file'] == 'OpenKIM':
            text = "Use the OpenKIM potential '{potentials}'"
        else:
            text = (
                "Read the forcefield file '{forcefield_file}' "
                "and use the {forcefield} forcefield."
            )

        return self.header + '\n' + __(
            text,
            indent=4 * ' ',
            **P,
        ).__str__()

    def run(self):
        """Setup the forcefield
        """

        next_node = super().run(printer=printer)

        # The options from command line, config file ...
        o = self.options  # noqa: F841

        P = self.parameters.current_values_to_dict(
            context=seamm.flowchart_variables._data
        )

        printer.important(__(self.header, indent=self.indent))

        if P['forcefield_file'] == 'OpenKIM':
            printer.important(
                __(
                    "Using the OpenKIM potential '{potentials}'",
                    **P,
                    indent=self.indent + '    '
                )
            )
            data.forcefield = 'OpenKIM'
            data.OpenKIM_Potential = P['potentials']
        else:
            printer.important(
                __(
                    "Reading the forcefield file '{forcefield_file}'",
                    **P,
                    indent=self.indent + '    '
                )
            )

            # Find the forcefield file
            path = pkg_resources.resource_filename(__name__, 'data/')
            ff_file = os.path.join(path, P['forcefield_file'])

            if P['forcefield'] == 'default':
                data.forcefield = seamm_ff_util.Forcefield(ff_file)
                printer.important(
                    __(
                        "   Using the default forcefield '{ff}'.",
                        ff=data.forcefield.forcefields[0],
                        indent=self.indent + '    '
                    )
                )
            else:
                data.forcefield = seamm_ff_util.Forcefield(
                    ff_file, P['forcefield']
                )
                printer.important(
                    __(
                        "   Using the forcefield '{forcefield}'",
                        **P,
                        indent=self.indent + '    '
                    )
                )

            data.forcefield.initialize_biosym_forcefield()
        printer.important('')

        return next_node
